#-*- coding:utf-8 -*-
"""
Module containing Exceptions classes.
"""

class RPCError(Exception):
    """
    Generic RPC Exception.
    """

class TimeoutError(Exception):
    """
    Dummy TimeoutError.
    """
